@@ -632,3 +632,311 @@ fn main() {
         })
     )
 }
+
+#[test]
+fn goto_definition_type() {
+    let code = "
+pub type Rec {
+  Var1(Int)
+  Var2(Int, Int)
+}
+
+pub fn make_var() -> Rec {
+  Var1(1)
+}";
+
+    assert_eq!(
+        definition(TestProject::for_source(code), Position::new(6, 22)),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\src\app.gleam"
+            } else {
+                "/src/app.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 1,
+                    character: 0
+                },
+                end: Position {
+                    line: 1,
+                    character: 12
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_type_in_module() {
+    let hex_src = "
+pub type Rec {
+  Var1(Int)
+  Var2(Int, Int)
+}
+";
+
+    let code = "
+import example_module
+fn make_var() -> example_module.Rec {
+  example_module.Var1(1)
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_hex_module("example_module", hex_src),
+            Position::new(2, 33)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\build\packages\hex\src\example_module.gleam"
+            } else {
+                "/build/packages/hex/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 1,
+                    character: 0
+                },
+                end: Position {
+                    line: 1,
+                    character: 12
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_type_in_path_dep() {
+    let dep = "
+pub type Rec {
+  Var1(Int)
+  Var2(Int, Int)
+}
+";
+
+    let code = "
+import example_module
+fn make_var() -> example_module.Rec {
+  example_module.Var1(1)
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_dep_module("example_module", dep),
+            Position::new(2, 33)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\dep\src\example_module.gleam"
+            } else {
+                "/dep/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 1,
+                    character: 0
+                },
+                end: Position {
+                    line: 1,
+                    character: 12
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_deep_type_in_module() {
+    let hex_src = "
+pub type Wobble {
+  Wobble(Int)
+}
+
+pub type Wibble(a) {
+  Wibble(a)
+}
+
+pub type Wabble(a) {
+  Wabble(a)
+}
+";
+
+    let code = "
+import example_module
+fn make_var() -> example_module.Wabble(example_module.Wibble(example_module.Wobble)) {
+  example_module.Wabble(example_module.Wibble(example_module.Wobble(1)))
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_hex_module("example_module", hex_src),
+            Position::new(2, 80)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\build\packages\hex\src\example_module.gleam"
+            } else {
+                "/build/packages/hex/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 1,
+                    character: 0
+                },
+                end: Position {
+                    line: 1,
+                    character: 15
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_import() {
+    let code = "
+import example_module
+fn main() {
+  example_module.my_num
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_module("example_module", "pub const my_num = 1"),
+            Position::new(1, 13)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\src\example_module.gleam"
+            } else {
+                "/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 0,
+                    character: 0
+                },
+                end: Position {
+                    line: 0,
+                    character: 0
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_import_aliased() {
+    let code = "
+import example_module as example
+fn main() {
+  example.my_num
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_module("example_module", "pub const my_num = 1"),
+            Position::new(1, 29)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\src\example_module.gleam"
+            } else {
+                "/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 0,
+                    character: 0
+                },
+                end: Position {
+                    line: 0,
+                    character: 0
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_import_unqualified_value() {
+    let code = "
+import example_module.{my_num}
+fn main() {
+  my_num
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_module("example_module", "pub const my_num = 1"),
+            Position::new(1, 26)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\src\example_module.gleam"
+            } else {
+                "/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 0,
+                    character: 10
+                },
+                end: Position {
+                    line: 0,
+                    character: 16
+                }
+            }
+        })
+    )
+}
+
+#[test]
+fn goto_definition_import_unqualified_type() {
+    let code = "
+import example_module.{type MyType}
+fn main() -> MyType {
+  0
+}
+";
+
+    assert_eq!(
+        definition(
+            TestProject::for_source(code).add_module("example_module", "pub type MyType = Int"),
+            Position::new(1, 33)
+        ),
+        Some(Location {
+            uri: Url::from_file_path(Utf8PathBuf::from(if cfg!(target_family = "windows") {
+                r"\\?\C:\src\example_module.gleam"
+            } else {
+                "/src/example_module.gleam"
+            }))
+            .unwrap(),
+            range: Range {
+                start: Position {
+                    line: 0,
+                    character: 0
+                },
+                end: Position {
+                    line: 0,
+                    character: 21
+                }
+            }
+        })
+    )
+}
