@@ -1,5 +1,4 @@
 use super::*;
-use crate::ast::TodoKind;
 use crate::{assert_no_warnings, assert_warning, assert_warnings_with_imports};
 
 #[test]
@@ -14,16 +13,7 @@ fn unknown_label() {
 
 #[test]
 fn todo_warning_test() {
-    assert_warning!(
-        "fn main() { 1 == todo }",
-        Warning::Todo {
-            kind: TodoKind::Keyword,
-            location: SrcSpan { start: 17, end: 21 },
-            typ: Arc::new(Type::Var {
-                type_: Arc::new(RefCell::new(TypeVar::Link { type_: int() })),
-            }),
-        },
-    );
+    assert_warning!("pub fn main() { 1 == todo }");
 }
 
 // https://github.com/gleam-lang/gleam/issues/1669
@@ -48,8 +38,9 @@ fn todo_with_known_type() {
 #[test]
 fn empty_func_warning_test() {
     assert_warning!(
-        "pub fn main() { foo() }
-        pub fn foo() { }"
+        "pub fn main() { wibble() }
+pub fn wibble() { }
+"
     );
 }
 
@@ -57,8 +48,8 @@ fn empty_func_warning_test() {
 fn warning_variable_never_used_test() {
     assert_warning!(
         "
-pub fn foo() { Ok(5) }
-pub fn main() { let five = foo() }"
+pub fn wibble() { Ok(5) }
+pub fn main() { let five = wibble() }"
     );
 }
 
@@ -80,14 +71,11 @@ fn result_discard_warning_test() {
     // Implicitly discarded Results emit warnings
     assert_warning!(
         "
-fn foo() { Ok(5) }
-fn main() {
-		foo()
-		5
-}",
-        Warning::ImplicitlyDiscardedResult {
-            location: SrcSpan { start: 34, end: 39 }
-        }
+pub fn wibble() { Ok(5) }
+pub fn main() {
+  wibble()
+  5
+}"
     );
 }
 
@@ -96,42 +84,29 @@ fn result_discard_warning_test2() {
     // Explicitly discarded Results do not emit warnings
     assert_no_warnings!(
         "
-pub fn foo() { Ok(5) }
-pub fn main() { let _ = foo() 5 }",
+pub fn wibble() { Ok(5) }
+pub fn main() { let _ = wibble() 5 }",
     );
 }
 
 #[test]
 fn unused_int() {
-    assert_warning!(
-        "fn main() { 1 2 }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 12, end: 13 }
-        }
-    );
+    assert_warning!("pub fn main() { 1 2 }");
 }
 
 #[test]
 fn unused_float() {
-    assert_warning!(
-        "fn main() { 1.0 2 }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 12, end: 15 }
-        }
-    );
+    assert_warning!("pub fn main() { 1.0 2 }");
 }
 
 #[test]
 fn unused_string() {
     assert_warning!(
         "
-    fn main() {
+    pub fn main() {
         \"1\"
-				2
-    }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 25, end: 28 }
-        }
+                2
+    }"
     );
 }
 
@@ -139,13 +114,10 @@ fn unused_string() {
 fn unused_bit_array() {
     assert_warning!(
         "
-    fn main() {
+    pub fn main() {
         <<3>>
 				2
-    }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 25, end: 30 }
-        }
+    }"
     );
 }
 
@@ -153,13 +125,10 @@ fn unused_bit_array() {
 fn unused_tuple() {
     assert_warning!(
         "
-    fn main() {
+    pub fn main() {
         #(1.0, \"Hello world\")
 				2
-    }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 25, end: 46 }
-        }
+    }"
     );
 }
 
@@ -167,13 +136,10 @@ fn unused_tuple() {
 fn unused_list() {
     assert_warning!(
         "
-    fn main() {
+    pub fn main() {
         [1, 2, 3]
 				2
-    }",
-        Warning::UnusedLiteral {
-            location: SrcSpan { start: 25, end: 34 }
-        }
+    }"
     );
 }
 
@@ -205,13 +171,7 @@ fn record_update_warnings_test2() {
             let past = Person(\"Quinn\", 27)
             let present = Person(..past)
             present
-        }",
-        Warning::NoFieldsRecordUpdate {
-            location: SrcSpan {
-                start: 182,
-                end: 196
-            }
-        }
+        }"
     );
 }
 
@@ -227,27 +187,14 @@ fn record_update_warnings_test3() {
             let past = Person(\"Quinn\", 27)
             let present = Person(..past, name: \"Quinn\", age: 28)
             present
-        }",
-        Warning::AllFieldsRecordUpdate {
-            location: SrcSpan {
-                start: 182,
-                end: 220
-            }
-        }
+        }"
     );
 }
 
 #[test]
 fn unused_private_type_warnings_test() {
     // External type
-    assert_warning!(
-        "type X",
-        Warning::UnusedType {
-            name: "X".into(),
-            location: SrcSpan { start: 0, end: 6 },
-            imported: false
-        }
-    );
+    assert_warning!("type X");
 }
 
 #[test]
@@ -258,14 +205,7 @@ fn unused_private_type_warnings_test2() {
 #[test]
 fn unused_private_type_warnings_test3() {
     // Type alias
-    assert_warning!(
-        "type X = Int",
-        Warning::UnusedType {
-            name: "X".into(),
-            location: SrcSpan { start: 0, end: 12 },
-            imported: false
-        }
-    );
+    assert_warning!("type X = Int");
 }
 
 #[test]
@@ -281,14 +221,7 @@ fn unused_private_type_warnings_test5() {
 #[test]
 fn unused_private_type_warnings_test6() {
     // Custom type
-    assert_warning!(
-        "type X { X }",
-        Warning::UnusedConstructor {
-            name: "X".into(),
-            location: SrcSpan { start: 9, end: 10 },
-            imported: false
-        }
-    );
+    assert_warning!("type X { X }");
 }
 
 #[test]
@@ -313,13 +246,7 @@ pub fn a() {
 
 #[test]
 fn unused_private_fn_warnings_test() {
-    assert_warning!(
-        "fn a() { 1 }",
-        Warning::UnusedPrivateFunction {
-            name: "a".into(),
-            location: SrcSpan { start: 0, end: 6 },
-        }
-    );
+    assert_warning!("fn a() { 1 }");
 }
 
 #[test]
@@ -334,13 +261,7 @@ fn used_private_fn_warnings_test2() {
 
 #[test]
 fn unused_private_const_warnings_test() {
-    assert_warning!(
-        "const a = 1",
-        Warning::UnusedPrivateModuleConstant {
-            name: "a".into(),
-            location: SrcSpan { start: 6, end: 7 },
-        }
-    );
+    assert_warning!("const a = 1");
 }
 
 #[test]
@@ -356,13 +277,7 @@ fn used_private_const_warnings_test2() {
 #[test]
 fn unused_variable_warnings_test() {
     // function argument
-    assert_warning!(
-        "pub fn a(b) { 1 }",
-        Warning::UnusedVariable {
-            name: "b".into(),
-            location: SrcSpan { start: 9, end: 10 },
-        }
-    );
+    assert_warning!("pub fn a(b) { 1 }");
 }
 
 #[test]
@@ -373,13 +288,7 @@ fn used_variable_warnings_test() {
 #[test]
 fn unused_variable_warnings_test2() {
     // Simple let
-    assert_warning!(
-        "pub fn a() { let b = 1 5 }",
-        Warning::UnusedVariable {
-            name: "b".into(),
-            location: SrcSpan { start: 17, end: 18 },
-        }
-    );
+    assert_warning!("pub fn a() { let b = 1 5 }");
 }
 
 #[test]
@@ -389,13 +298,7 @@ fn used_variable_warnings_test2() {
 
 #[test]
 fn unused_variable_shadowing_test() {
-    assert_warning!(
-        "pub fn a() { let b = 1 let b = 2 b }",
-        Warning::UnusedVariable {
-            name: "b".into(),
-            location: SrcSpan { start: 17, end: 18 },
-        }
-    );
+    assert_warning!("pub fn a() { let b = 1 let b = 2 b }");
 }
 
 #[test]
@@ -406,13 +309,7 @@ fn used_variable_shadowing_test() {
 #[test]
 fn unused_destructure() {
     // Destructure
-    assert_warning!(
-        "pub fn a(b) { case b { #(c, _) -> 5 } }",
-        Warning::UnusedVariable {
-            name: "c".into(),
-            location: SrcSpan { start: 25, end: 26 },
-        }
-    );
+    assert_warning!("pub fn a(b) { case b { #(c, _) -> 5 } }");
 }
 
 #[test]
@@ -423,49 +320,25 @@ fn used_destructure() {
 #[test]
 fn unused_imported_module_warnings_test() {
     assert_warning!(
-        ("gleam/foo", "pub fn bar() { 1 }"),
-        "import gleam/foo",
-        Warning::UnusedImportedModule {
-            name: "foo".into(),
-            location: SrcSpan { start: 0, end: 16 },
-        }
+        ("gleam/wibble", "pub fn wobble() { 1 }"),
+        "import gleam/wibble"
     );
 }
 
 #[test]
 fn unused_imported_module_with_alias_warnings_test() {
     assert_warning!(
-        ("gleam/foo", "pub fn bar() { 1 }"),
-        "import gleam/foo as bar",
-        Warning::UnusedImportedModule {
-            name: "bar".into(),
-            location: SrcSpan { start: 0, end: 23 },
-        }
+        ("gleam/wibble", "pub fn wobble() { 1 }"),
+        "import gleam/wibble as wobble"
     );
 }
 
 // https://github.com/gleam-lang/gleam/issues/2326
 #[test]
 fn unused_imported_module_with_alias_and_unqualified_name_warnings_test() {
-    let warnings = get_warnings(
-        "import gleam/one.{two} as three",
-        vec![("thepackage", "gleam/one", "pub fn two() { 1 }")],
-    );
-    assert!(!warnings.is_empty());
-    assert_eq!(
-        Warning::UnusedImportedValue {
-            name: "two".into(),
-            location: SrcSpan { start: 18, end: 21 },
-        },
-        warnings[0]
-    );
-    assert_eq!(
-        Warning::UnusedImportedModuleAlias {
-            alias: "three".into(),
-            location: SrcSpan { start: 23, end: 31 },
-            module_name: "gleam/one".into()
-        },
-        warnings[1]
+    assert_warning!(
+        ("thepackage", "gleam/one", "pub fn two() { 1 }"),
+        "import gleam/one.{two} as three"
     );
 }
 
@@ -473,39 +346,52 @@ fn unused_imported_module_with_alias_and_unqualified_name_warnings_test() {
 fn unused_imported_module_with_alias_and_unqualified_name_no_warnings_test() {
     assert_warning!(
         ("package", "gleam/one", "pub fn two() { 1 }"),
-        "import gleam/one.{two} as three\npub fn baz() { two() }"
+        "import gleam/one.{two} as three\npub fn wibble() { two() }"
     );
 }
 
 #[test]
 fn unused_imported_module_no_warning_on_used_function_test() {
     assert_no_warnings!(
-        ("thepackage", "gleam/foo", "pub fn bar() { 1 }"),
-        "import gleam/foo pub fn baz() { foo.bar() }",
+        ("thepackage", "gleam/wibble", "pub fn wobble() { 1 }"),
+        "import gleam/wibble pub fn wibble() { wibble.wobble() }",
     );
 }
 
 #[test]
 fn unused_imported_module_no_warning_on_used_type_test() {
     assert_no_warnings!(
-        ("thepackage", "gleam/foo", "pub type Foo = Int"),
-        "import gleam/foo pub fn baz(a: foo.Foo) { a }",
+        ("thepackage", "gleam/wibble", "pub type Wibble = Int"),
+        "import gleam/wibble pub fn wibble(a: wibble.Wibble) { a }",
     );
 }
 
 #[test]
 fn unused_imported_module_no_warning_on_used_unqualified_function_test() {
     assert_no_warnings!(
-        ("thepackage", "gleam/foo", "pub fn bar() { 1 }"),
-        "import gleam/foo.{bar} pub fn baz() { bar() }",
+        ("thepackage", "gleam/wibble", "pub fn wobble() { 1 }"),
+        "import gleam/wibble.{wobble} pub fn wibble() { wobble() }",
     );
 }
 
 #[test]
 fn unused_imported_module_no_warning_on_used_unqualified_type_test() {
     assert_no_warnings!(
-        ("thepackage", "gleam/foo", "pub type Foo = Int"),
-        "import gleam/foo.{type Foo} pub fn baz(a: Foo) { a }",
+        ("thepackage", "gleam/wibble", "pub type Wibble = Int"),
+        "import gleam/wibble.{type Wibble} pub fn wibble(a: Wibble) { a }",
+    );
+}
+
+// https://github.com/gleam-lang/gleam/issues/3313
+#[test]
+fn imported_module_with_alias_no_warning_when_only_used_in_case_test() {
+    assert_no_warnings!(
+        (
+            "thepackage",
+            "gleam/wibble",
+            "pub type Wibble { Wibble(Int) }"
+        ),
+        "import gleam/wibble as f\npub fn wibble(a) { case a { f.Wibble(int) -> { int } }  }",
     );
 }
 
@@ -1088,66 +974,48 @@ fn const_bytes_option() {
 #[test]
 fn unused_module_wuth_alias_warning_test() {
     assert_warning!(
-        ("gleam/foo", "pub const one = 1"),
-        "import gleam/foo as bar",
-        Warning::UnusedImportedModule {
-            name: "bar".into(),
-            location: SrcSpan { start: 0, end: 23 },
-        }
+        ("gleam/wibble", "pub const one = 1"),
+        "import gleam/wibble as wobble"
     );
 }
 
 #[test]
 fn unused_alias_warning_test() {
     assert_warnings_with_imports!(
-        ("gleam/foo", "pub const one = 1");
+        ("gleam/wibble", "pub const one = 1");
         r#"
-            import gleam/foo.{one} as bar
+            import gleam/wibble.{one} as wobble
             const one = one
         "#,
-        Warning::UnusedPrivateModuleConstant {
-            name: "one".into(),
-            location: SrcSpan { start: 61, end: 64 },
-        },
-        Warning::UnusedImportedModuleAlias {
-            alias:"bar".into(),
-            location: SrcSpan { start: 36, end: 42 },
-            module_name: "gleam/foo".into(),
-        }
     );
 }
 
 #[test]
 fn used_type_with_import_alias_no_warning_test() {
     assert_no_warnings!(
-        ("gleam", "gleam/foo", "pub const one = 1"),
-        "import gleam/foo as _bar"
+        ("gleam", "gleam/wibble", "pub const one = 1"),
+        "import gleam/wibble as _wobble"
     );
 }
 
 #[test]
 fn discarded_module_no_warnings_test() {
-    assert_no_warnings!(("gleam", "foo", "pub const one = 1"), "import foo as _bar");
+    assert_no_warnings!(
+        ("gleam", "wibble", "pub const one = 1"),
+        "import wibble as _wobble"
+    );
 }
 
 #[test]
 fn unused_alias_for_duplicate_module_no_warning_for_alias_test() {
     assert_warnings_with_imports!(
-        ("a/foo", "pub const one = 1"),
-        ("b/foo", "pub const two = 2");
+        ("a/wibble", "pub const one = 1"),
+        ("b/wibble", "pub const two = 2");
         r#"
-            import a/foo
-            import b/foo as bar
-            const one = foo.one
+            import a/wibble
+            import b/wibble as wobble
+            const one = wibble.one
         "#,
-        Warning::UnusedPrivateModuleConstant {
-            name: "one".into(),
-            location: SrcSpan { start: 76, end: 79 },
-        },
-        Warning::UnusedImportedModule {
-            name: "bar".into(),
-            location: SrcSpan { start: 38, end: 57 },
-        }
     );
 }
 
@@ -1160,10 +1028,7 @@ pub fn main(x) {
     _ -> Error(Nil)
   }
   Nil
-}",
-        Warning::ImplicitlyDiscardedResult {
-            location: SrcSpan { start: 20, end: 52 }
-        }
+}"
     );
 }
 
@@ -2012,3 +1877,95 @@ fn warnings_for_matches_on_literal_values_that_are_not_like_an_if_2() {
         "#
     );
 }
+
+#[test]
+fn redundant_function_capture_in_pipe_1() {
+    assert_warning!(
+        "
+  pub fn wibble(_, _) { 1 }
+
+  pub fn main() {
+    1 |> wibble(_, 2) |> wibble(2)
+  }
+"
+    );
+}
+
+#[test]
+fn redundant_function_capture_in_pipe_2() {
+    assert_warning!(
+        "
+  pub fn wobble(_) { 1 }
+
+  pub fn main() {
+    1 |> wobble(_) |> wobble
+  }
+"
+    );
+}
+
+#[test]
+fn redundant_function_capture_in_pipe_3() {
+    assert_warning!(
+        "
+  pub fn wobble(_) { 1 }
+
+  pub fn main() {
+    1 |> wobble |> wobble(_)
+  }
+"
+    );
+}
+
+#[test]
+fn redundant_function_capture_in_pipe_4() {
+    assert_warning!(
+        "
+  pub fn wibble(_, _) { 1 }
+
+  pub fn main() {
+    1 |> wibble(2) |> wibble(_, 2)
+  }
+"
+    );
+}
+
+#[test]
+fn redundant_function_capture_in_pipe_5() {
+    assert_no_warnings!(
+        "
+  pub fn wibble(_, _) { 1 }
+
+  pub fn main() {
+    1 |> wibble(2, _)
+  }
+"
+    );
+}
+
+#[test]
+fn deprecated_list_append_syntax() {
+    assert_warning!(
+        r#"
+    pub fn main() {
+      let letters = ["b", "c"]
+      ["a"..letters]
+    }
+        "#
+    );
+}
+
+#[test]
+fn deprecated_list_pattern_syntax() {
+    assert_warning!(
+        r#"
+    pub fn main() {
+      let letters = ["b", "c"]
+      case letters {
+        ["a"..rest] -> rest
+        _ -> []
+      }
+    }
+        "#
+    );
+}
